function [patterns, W, aux ] = MDA(patterns, targets)
% MDA: Reshape the data points using Multiple Discriminant Analysis.
%
% Inputs:
%	train_patterns	- matrix of patterns.
%                         of dimension = [ number of features, number of training patterns ].
%	train_targets	- vector of pattern labels.
%                         each element of the vector = training patterns label.
%
% Outputs:
%	patterns        - New matrix patterns
%                         of dimension = [ number of classes - 1, number of training patterns ].
%	W               - Reshape martix
%                         of dimension = [ number of features, number of classes - 1 ];
%       aux             - Auxiliary structure with intermediate calculations (if needed)
% 
% Written (2004-10-01) by John Weatherwax (wax@alumni.mit.edu)
%-----
  
% To prevent the production of complex eigenvalues in the "eig" routine below
% we regularize (by diagonal loading) by a small amount. 
ABS_EPS = 10^(-10);   

[d,n]   = size(patterns);
tLabels = unique(targets);
c       = length(tLabels);

if(d < c-1),
  error( 'Pattern space dimension is not significantly greater than the number of classes' );
end
if( prod(size(targets)) ~= length(targets) )
  error( 'Input ''targets'' is not a one dimensional array' );
end

%
% Compute m (pooled sample mean):
% 
m = mean(patterns.').';

%
% Compute n_i (number of samples of class index i) and
%         m_i (class means):
% 
nI = zeros(1,c);
mI = zeros(d,c);
for il=1:c,
  it = find(targets==tLabels(il));
  nI(il) = length(it);
  mI(:,il) = mean(patterns(:,it).').';
end

%
% Compute S_i (class specific scatter matrix):
% 
S_i = zeros(d,d,c);
for il=1:c,
  it = find(targets==tLabels(il));
  S_i(:,:,il) = (d-1)*cov(patterns(:,it).');
end

%
% Calculate S_W (within-class scatter matrix): 
%
S_W = zeros(d,d);
for il=1:c,
  S_W = S_W + S_i(:,:,il);
end

%
% Calculate S_B (between-class scatter matrix):
%
S_B = zeros(d,d);
for il=1:c,
  S_B = S_B + nI(il)*(mI(:,il)-m)*((mI(:,il)-m).');
end

% Diagonal loading regularization of Sb, and Sw:
if( 1 ) 
  S_B = S_B + ABS_EPS*eye(size(S_B)); 
  S_W = S_W + ABS_EPS*eye(size(S_W)); 
end

% Compute the generalized eigenvalues and eigenvectors:
[V, D] = eig(S_B,S_W);

% Further orthogonalize the columns of V:
Q = gram(V); V = Q; 
% $$$ % Normalize the columns of V: 
% $$$ for iv=1:size(V,1)
% $$$   nv = sqrt(dot(V(:,iv),V(:,iv)));
% $$$   V(:,iv) = V(:,iv)/nv; 
% $$$ end

% Extract and sort the eigenvalues in decending order:
[sDum,iSort] = sort(-diag(D));

% Extract eigenvectors with largest eigenvalues:
W = V(:,iSort(1:c-1));

% Calculate new patterns using this transformation:
patterns = W.' * patterns;

% Pass out any information that might be useful latter:
aux.m   = m;
aux.mI  = mI;
aux.nI  = nI;
aux.S_i = S_i;
aux.S_W = S_W;
aux.S_B = S_B;
