/*--------------------------------------------------------------
// file: dMIdW.c - compute the derivative of the mutual information 
//       with respect to the matrix of transformation W
//   dMIdW = dMIdW(dIdY,Xs)
// INPUTS:
//   dIdY - Nxd matrix (N=number of data points, d=reduced feature dimension size)
//   Xs   - NxD matrix (N=number of data points, D=full feature dimension size)
// OUTPUTS:
//   dMIdW - matrix of size D x d representing a linear transformation from Xs to Ys
//
// Checked:
//
//  Written by:
//  -- 
//  John L. Weatherwax                2005-10-30
//  
//  email: wax@alum.mit.edu
//  
//  Please send comments and especially bug reports to the
//  above email address.
//  
//--------------------------------------------------------------*/
#include <math.h>
#include "mex.h"

/*--------------------------------------------------------------
// function: mexFunction - Entry point from Matlab environment
// INPUTS:
//   nlhs - number of left hand side arguments (outputs)
//   plhs[] - pointer to table where created matrix pointers are
//            to be placed
//   nrhs - number of right hand side arguments (inputs)
//   prhs[] - pointer to table of input matrices
//--------------------------------------------------------------*/
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  int N,d,D; 
  int k,i,m,n,colLoc; /* Loop variables */
  double k_sum,i_sum,dIdyil,dyidWmn,dWmndTk; /* dummy variables */
  double *pdIdY,*pXs,*pdWdT,*pdMIdW;

  if (nrhs < 2)
    mexErrMsgTxt("Not enough input arguments.");
  if (nrhs > 2)
    mexErrMsgTxt("To many input arguments.");
  if (nlhs < 1)
    mexErrMsgTxt("Too few output arguments.");
  if (nlhs > 2)
    mexErrMsgTxt("Too many output arguments.");
  if (!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]))
    mexErrMsgTxt("Input arguments must be type double.");
  if (mxGetM(prhs[0]) != mxGetM(prhs[1])) /* This doesn't seem to work ... code crashes but this should catch it ... */ 
    mexErrMsgTxt("number of rows of argument 1 must equal that of argument 2.");
  if (mxIsComplex(prhs[0]) || mxIsComplex(prhs[1]))
    mexWarnMsgTxt("Complex parts ignored.");

  /* dMIdW(Nxd,NxD) => D x d */ 
  N = mxGetM(prhs[0]); 
  d = mxGetN(prhs[0]); 
  D = mxGetN(prhs[1]); 

  /* Create output matrix */ 
  plhs[0] = mxCreateDoubleMatrix(D,d,mxREAL);

  pdIdY  = mxGetPr(prhs[0]);
  pXs    = mxGetPr(prhs[1]);
  pdMIdW = (double *)mxGetData(plhs[0]);

  /* Now the computation */
  for( m=0; m<D; m++ ){ /* Loop over each of the (m,n) components of W */ 
    for( n=0; n<d; n++ ){
      i_sum = 0; 
      for( i=0; i<N; i++ ){ /* Loop over each sample point */
        i_sum += pXs[i+N*m] * pdIdY[i+N*n]; 
      } /* i */ 
      pdMIdW[m+D*n] = i_sum;
    } /* n */ 
  } /* m */ 
} /* end mexFunction() */
